;**********************************************************************************
;Title:			stepper motor controller code
;Version:			1.0
;Last update:		2002.02.23
;Target MCU:		PIC16F628
;support E-mail:		mindtan@yahoo.com
;support website:		www.geocities.com/mindtan2000.html
;Register Usage:		3
;Interrupt Usage:		External Interrupt (INT)
;total memory space
;after compile to hex:	73 word (1 word = 14bit)
;Total Flash Program:	2048 X 1 word (2048 X 14 bit)
;
;Description
;
;This source code shown how to make a simple stepper motor controller using external
;interrupt method (pin RB0/INT). The port B (bit 4 to 7) used as step sequence
;pins. The step sequence is simple full step with one wave algorithm only.
;Becareful! do not program Pic16F628 under low programming mode which will
;cause bit 4 of PORT B set as input pin only.
;**********************************************************************************
	LIST P=16F628		; list directive to define processor
	ERRORLEVEL -302
	include <p16F628.inc>	; processor specific variable definitions
	page
	__CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;******************************************************************************
;	EQUATES/ DEFINITIONS
;******************************************************************************
SCK	equ	0
DIR	equ	1

set_output		equ	0x20

W_SAVE		EQU	0X21
STATUS_SAVE		EQU	0x22

;******************************************************************************
;main program start here
;******************************************************************************

	org	00h		; reset vector
	goto	start
	ORG	0X004		;interrupt vector
	GOTO	INT_SERV

;******************************************************************************
;	MAIN ROUTINE
;******************************************************************************
start	
	call	Port_Init		;call hardware setting subrotine
reloop
	bsf	OPTION_REG, INTEDG	;select interrupt on rising edge of RB0/INT pin
	BSF	INTCON, INTE	;enable RB0/INT external interrupt
	BSF	INTCON, GIE
	goto	reloop		

;----------------------------------------------------------------------------
;SUB: INTERRUPT VECTOR
;----------------------------------------------------------------------------
INT_SERV
	MOVWF		W_SAVE
	SWAPF		STATUS, W
	MOVWF		STATUS_SAVE
	BCF 		STATUS, RP0
	btfsc		PORTB, DIR
	goto		forward
	goto		reverse
forward
	call		run_forward
	goto		exit
reverse
	call		run_reverse
exit		
	BCF		INTCON, INTF
	SWAPF		STATUS_SAVE, W
	MOVWF		STATUS
	SWAPF		W_SAVE, F
	SWAPF		W_SAVE, W
	RETFIE


;*********************************************************************************
;subrotine : stepper motor (full step, one wave)
;*********************************************************************************
run_forward
	bcf		STATUS, C
	movf		set_output, w
	iorlw 	b'00000000'
	btfsc		STATUS, Z
	goto		reset_output_f
	movf		set_output, w
	xorlw 	b'10000000'
	btfss		STATUS, Z
	goto		rlf_output
	goto		reset_output_f
rlf_output	
	rlf		set_output, f
	movf		set_output, w
	movwf		PORTB
	return
reset_output_f
	movlw		b'00010000'
	movwf		set_output
	movwf		PORTB
	return

run_reverse
	bcf		STATUS, C
	movf		set_output, w
	iorlw 	b'00000000'
	btfsc		STATUS, Z
	goto		reset_output_r
	movf		set_output, w
	xorlw 	b'00010000'
	btfss		STATUS, Z
	goto		rrf_output
	goto		reset_output_r
rrf_output	
	rrf		set_output, f
	movf		set_output, w
	movwf		PORTB
	return
reset_output_r
	movlw		b'10000000'
	movwf		set_output
	movwf		PORTB
	return


;******************************************************************************
;Subroutine : Initialization of hardware and Port setting
;******************************************************************************
Port_Init	
	clrf	STATUS	; choose page 0 (Blank 0)
	bsf	STATUS, RP0	; select memory register page 1 (Blank 1)
	movlw	0x0f		;value used to intialize data direction
	movwf	TRISB		;set bit 0-3 (RB4 to RB7) as input and 
				;bit 4-7 (RB4 to RB7) as output fot PORT B
	bcf	STATUS, RP0	; go back to register page 0 (Blank 0)
	clrf	PORTB
	movlw	0x10
	movwf	set_output
	movwf	PORTB
	return

	end





















